#!/usr/bin/env python
#   NOtraining.py
#   Train recognition of 4 objects in a visual search paradigm
#   EGC October 2018
from PIL import Image
import numpy as np
import os, glob, sys, math, pickle, random
from itertools import combinations as comb
from psychopy import visual, core, data, event, logging, clock
from numpy.random import randint, normal, shuffle
from psychopy.core import getTime

def NOtraining(subID,session):
    # Experiment parameters
    maxRT=1.50
    imgDir="stimuli/"
    outDir="results/"
    lListsDir="learningLists/"
    # Quit if files already exists
    dfile=str(outDir + "NOtraining_" + subID + "_session" + str(session) + ".txt")
    if os.path.exists(dfile):
        print("\n\n\nIt looks like this session was already completed. Did you input the correct session and subject number?\n\n\n\n")
        core.quit()  # abort experiment
    # Stim and Directories
    if session < 11:
        l=open(str(lListsDir + subID + '_1.txt'),'r')
    elif session > 10:
        l=open(str(lListsDir + subID + '_2.txt'),'r')
    sList=l.read().splitlines()
    stimDir=[]
    for file in os.listdir(imgDir):
        if file.endswith("Store"):
            print ('')#do nothing
        else: 
            stimDir.append(file)
    
    # Create procedure and dir list
    pList=[]
    pList[0:59]=np.ones(60)-1
    pList[60:119]=np.ones(60)
    pList[120:179]=np.ones(60)+1
    pList[180:239]=np.ones(60)+2
    pList=[int(x) for x in pList]
    pList=[[x] for x in pList]
    random.shuffle(pList)
    for i in range(0,240):
        # Choose distractor stim
        a=np.array([0,1,2,3,4,5,6,7,8,9,10,11])
        c=pList[i][0]
        a=np.setdiff1d(a,c)
        dStim=np.random.choice(a,size=3,replace=False)
        dStim=np.ndarray.tolist(dStim)
        dStim.append(c)
        random.shuffle(dStim)
        # Choose image angles, no overlap
        p=np.array([0,1,2,3,4,5,6,7,8])
        posStim=np.random.choice(p,size=5,replace=False)
        posStim=np.ndarray.tolist(posStim)
        pList[i].append(dStim)
        pList[i].append(posStim)
    
    # Set up output file
    outF=open(outDir + "NOtraining_" + subID + "_session" + str(session) + ".txt", "w")
    outF.write("subID\timageTarget\tdisNO1\tdisNO2\tdisNO3\tdisNO4\tsession\ttrial\taccuracy\tRT\n")
    #create a window
    win1 = visual.Window([1440,900], monitor="testMonitor", units="deg", fullscr=True)
    win1.mouseVisible = False
    # display instructions and wait
    message1 = visual.TextStim(win1, pos=[0,+8],text="Welcome to the Perceptual Training Experiment", wrapWidth=50)
    message2 = visual.TextStim(win1, pos=[0,+5],text="This experiment is designed to help you learn to recognize 4 specific objects", wrapWidth=50)
    message3 = visual.TextStim(win1, pos=[0,+3],text="You will see a target object in the middle of the screen", wrapWidth=50)
    message4 = visual.TextStim(win1, pos=[0,+1],text="followed by 4 images containing different objects.", wrapWidth=50)
    message5 = visual.TextStim(win1, pos=[0,-1],text="Using the arrow keys, pick the image that contains the object from the first image.", wrapWidth=50)
    message6 = visual.TextStim(win1, pos=[0,-5],text="You will earn points based on accuracy and reaction time.", wrapWidth=50)
    message7 = visual.TextStim(win1, pos=[0,-8],text="<Press 'space' to start the experiment>", wrapWidth=50)
    message1.draw()
    message2.draw()
    message3.draw()
    message4.draw()
    message5.draw()
    message6.draw()
    message7.draw()
    win1.flip()
    event.waitKeys(keyList=['space'])
    win1.flip(clearBuffer=True)
    core.wait(2.0)
    # Run Experiment #################################################################
    nTrials=240
    pos1=[0,7]
    pos2=[7,0]
    pos3=[0,-7]
    pos4=[-7,0]
    score=0
    for t in range(0,nTrials):
        if t!=0 and t % 80 == 0: # Take a break
            message10=visual.TextStim(win1, pos=[0,2],text="Take a break")
            board=str("Score: " + str(score))
            scr=visual.TextStim(win1, pos=[0,0],text=board)
            message11=visual.TextStim(win1, pos=[0,-2],text="<Press 'space' to continue>")
            scr.draw()
            message10.draw()
            message11.draw()
            win1.flip()
            event.waitKeys(keyList=['space'])
            win1.flip(clearBuffer=True)
            core.wait(1.0)
        
        # Load trial stimuli
        imgC=str(sList[pList[t][0]] + stimDir[pList[t][2][0]] + '.JPG')
        img1=str(sList[pList[t][1][0]] + stimDir[pList[t][2][1]] + '.JPG')
        img2=str(sList[pList[t][1][1]] + stimDir[pList[t][2][2]] + '.JPG')
        img3=str(sList[pList[t][1][2]] + stimDir[pList[t][2][3]] + '.JPG')
        img4=str(sList[pList[t][1][3]] + stimDir[pList[t][2][4]] + '.JPG')
        cDir=str(stimDir[pList[t][2][0]] + '/' + imgC)
        qDir=str(stimDir[pList[t][2][1]] + '/' + img1) 
        wDir=str(stimDir[pList[t][2][2]] + '/' + img2)
        eDir=str(stimDir[pList[t][2][3]] + '/' + img3)
        rDir=str(stimDir[pList[t][2][4]] + '/' + img4)
        stimC=visual.ImageStim(win1,image=imgDir +cDir,pos=[0,0])
        stim1=visual.ImageStim(win1,image=imgDir +qDir,size=(7,7), pos=[pos1])
        stim2=visual.ImageStim(win1,image=imgDir +wDir,size=(7,7), pos=[pos2])
        stim3=visual.ImageStim(win1,image=imgDir +eDir,size=(7,7), pos=[pos3])
        stim4=visual.ImageStim(win1,image=imgDir +rDir,size=(7,7), pos=[pos4])
        corPos=pList[t][1].index(pList[t][0])
        # Display stimulus 1: 200ms
        stimC.draw()
        win1.flip()
        core.wait(0.200)
        # Fixation cross:400ms
        fixation = visual.TextStim(win1, pos=[0,0],text="+")
        fixation.draw()
        win1.flip()
        core.wait(0.400)
        # Display stimulus 2: until response
        stim1.draw()
        stim2.draw()
        stim3.draw()
        stim4.draw()
        win1.flip()
        clock=getTime()
        resp=None
        while resp==None:
            allKeys=event.waitKeys(keyList=['up','down','right','left','escape'])
            for thisKey in allKeys:
                rt=getTime()-clock
                if thisKey=='up':
                    resp=0
                elif thisKey=='right':
                    resp=1
                elif thisKey=='down':
                    resp=2
                elif thisKey=='left':
                    resp=3
                elif thisKey in ['escape']:
                    core.quit()  # abort experiment
            event.clearEvents()  # clear other (eg mouse) events - they clog the buffer
        if corPos == resp:
            acc=1
            if rt < maxRT:
                inc= round(100+ math.pow((maxRT-rt),3)*25)
                score+=inc
                corrM=str("Correct. \n\n Points: " + str(inc))
                corm=visual.TextStim(win1, pos=[0,0],text=corrM)
                corm.draw()
                win1.flip()
            else:
                inc= 50
                score+=inc
                corrM=str("Correct, but too slow... \n\n Points: " + str(inc))
                corm=visual.TextStim(win1, pos=[0,0],text=corrM)
                corm.draw()
                win1.flip()
        else:
            acc=0
            inc=-t
            score+=inc
            corrM=str("Incorrect. \n\n Points: " + str(inc))
            corm=visual.TextStim(win1, pos=[0,0],text=corrM)
            corm.draw()
            win1.flip()
        core.wait(1.00)
        win1.flip(clearBuffer=True)
        outF.write( "%s\t%s\t%s\t%s\t%s\t%s\t%d\t%d\t%d\t%f\n " % (subID, imgC, img1, img2, img3, img4, session, t, acc, rt))
        core.wait(0.100) # Time between trials ==
        
    # Save data and close experiment #######################################################
    outF.close
    cm = visual.TextStim(win1, pos=[0,0],text="Great work! You're all done.")
    board=str("Total score: " + str(score))
    scr = visual.TextStim(win1, pos=[0,-3],text=board)
    scr.draw()
    cm.draw()
    win1.flip()
    core.wait(3.0)
    win1.close()

if __name__ == '__main__':
    subID = '205'                           #######str(sys.argv[1])
    session = 99                              ###########int(sys.argv[2])
    NOtraining(subID,session)
